// src/VidzoneSDK.js

class VidzoneSDK {
  constructor(userConsent = true) {
      this.nonceLoader = null;
      this.userConsent = userConsent;
  }

  async init() {
    if (typeof goog === 'undefined' || typeof goog.pal === 'undefined') {
      throw new Error('Google PAL SDK is not loaded.');
    }
    
    try {
        const resources = [
            this._loadResource({
                url: 'https://imasdk.googleapis.com/pal/sdkloader/pal.js',
                async: false,
                defer: false,
                container: document.body || document.head || document.documentElement
            })
        ];

        await Promise.all(resources);

        const consentSettings = new goog.pal.ConsentSettings();
        consentSettings.allowStorage = this.userConsent;

        this.nonceLoader = new goog.pal.NonceLoader(consentSettings);
    } catch (error) {
        throw new Error('Error during SDK initialization: ' + error);
    }
  }

  _loadResource(options) {
    return new Promise((resolve, reject) => {
        const element = document.createElement('script');
        if (options.async) element.async = true;
        if (options.defer) element.defer = true;
        element.src = options.url;
        element.onload = resolve;
        element.onerror = reject;
        options.container.appendChild(element);
    });
  }

  async generateNonce(params) {
    const request = new goog.pal.NonceRequest();
    request.adWillAutoPlay = params?.adWillAutoPlay;
    request.adWillPlayMuted = params?.adWillPlayMuted;
    request.continuousPlayback = true;
    request.descriptionUrl = '';
    request.iconsSupported = false;
    request.playerType = params?.playerType;
    request.playerVersion = params?.playerVersion;
    request.ppid = params.ppid;
    request.sessionId = params.sessionId;
    request.url = params?.pageUrl;
    request.videoHeight = params?.videoHeight;
    request.videoWidth = params?.videoWidth;

    try {
        const manager = await this.nonceLoader.loadNonceManager(request);
        return new VidzoneNonceManager(manager);
    } catch (error) {
        throw new Error('Error generating nonce: ' + error);
    }
  }
}

class VidzoneNonceManager {
  constructor(nonceManager) {
    this.nonceManager = nonceManager;
  }

  _log(msg) {
    if (localStorage.vidzonesdkdebug) {
        console.log('[VidzoneSDK Debug] ' + msg);
    }
  }

  getNonce() {
    this._log('getNonce called');
    return this.nonceManager.getNonce();
  }

  onPlaybackStart() {
    this._log('onPlaybackStart called');
    this.nonceManager?.sendPlaybackStart();
  }

  onPlaybackEnd() {
    this._log('onPlaybackEnd called');
    this.nonceManager?.sendPlaybackEnd();
  }

  getParams() {
    return this.getExtraParams();
  }

  getExtraParams() {
    this._log('getExtraParams called');
    return encodeURIComponent(btoa(JSON.stringify({
      givn: this.getNonce(),
    })))
  }
} 

module.exports = VidzoneSDK;
